from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .taxes_get_response_results_options import TaxesGetResponse_results_options
    from .taxes_get_response_results_summary import TaxesGetResponse_results_summary

@dataclass
class TaxesGetResponse_results(Parsable):
    # The unique identifier (UUID) of the object the tax is associated with.
    association_id: Optional[UUID] = None
    # The type of object the tax is associated with.
    association_type: Optional[str] = None
    # Not relevant
    container_id: Optional[UUID] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique identifier of the tax.
    id: Optional[UUID] = None
    # The name of the tax.Max length: 1024
    name: Optional[str] = None
    # Indicates whether the tax needs to be recalculated because the associated item was updated after the tax was applied.
    need_update: Optional[bool] = None
    # Not relevant
    options: Optional[TaxesGetResponse_results_options] = None
    # A consolidated report of all taxes applied within a specific project.
    summary: Optional[list[TaxesGetResponse_results_summary]] = None
    # Not relevant
    tax_formula_id: Optional[UUID] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TaxesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TaxesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TaxesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .taxes_get_response_results_options import TaxesGetResponse_results_options
        from .taxes_get_response_results_summary import TaxesGetResponse_results_summary

        from .taxes_get_response_results_options import TaxesGetResponse_results_options
        from .taxes_get_response_results_summary import TaxesGetResponse_results_summary

        fields: dict[str, Callable[[Any], None]] = {
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_str_value()),
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "needUpdate": lambda n : setattr(self, 'need_update', n.get_bool_value()),
            "options": lambda n : setattr(self, 'options', n.get_object_value(TaxesGetResponse_results_options)),
            "summary": lambda n : setattr(self, 'summary', n.get_collection_of_object_values(TaxesGetResponse_results_summary)),
            "taxFormulaId": lambda n : setattr(self, 'tax_formula_id', n.get_uuid_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_str_value("associationType", self.association_type)
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_bool_value("needUpdate", self.need_update)
        writer.write_object_value("options", self.options)
        writer.write_collection_of_object_values("summary", self.summary)
        writer.write_uuid_value("taxFormulaId", self.tax_formula_id)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .rfi_types_get_response_results_manager import RfiTypesGetResponse_results_manager
    from .rfi_types_get_response_results_project_coordinator import RfiTypesGetResponse_results_projectCoordinator
    from .rfi_types_get_response_results_project_reviewer import RfiTypesGetResponse_results_projectReviewer
    from .rfi_types_get_response_results_status import RfiTypesGetResponse_results_status
    from .rfi_types_get_response_results_watchers import RfiTypesGetResponse_results_watchers
    from .rfi_types_get_response_results_wf_type import RfiTypesGetResponse_results_wfType

@dataclass
class RfiTypesGetResponse_results(Parsable):
    # Not relevant
    bridge_target_project_ids: Optional[list[str]] = None
    # A list of predefined categories to assign to the RFI.Categories help group RFIs for filtering and reporting. Each value must match a category configured in the project’s RFI settings. Categories are case-sensitive and project-specific.RFI categories are configured in Project Admin and may differ between projects.Some possible values: ``Code Compliance``, ``Constructability``, ``Design Coordination``, ``Documentation Conflict``, ``Documentation Incomplete``, ``Field condition``, ``Other``.
    category: Optional[list[str]] = None
    # The default cost impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.
    cost_impact: Optional[str] = None
    # The list of available disciplines for RFIs.Each discipline is configured in Project Admin.Some possible values: ``Architectural``, ``Civil/Site``, ``Concrete``, ``Electrical``, ``Exterior Envelope``, ``Fire Protection``, ``Interior/Finishes``, ``Landscaping``, ``Masonry``, ``Mechanical``, ``Plumbing``, ``Structural``, ``Other``.
    discipline: Optional[list[str]] = None
    # The number of calendar days from the RFI creation date to the default due date. Used to automatically calculate the RFI due date when creating a new RFI of this type.
    due_date_offset: Optional[int] = None
    # The ID of the default RFI type assigned to the project. This is the unique identifier of the RFI type that will be selected by default when creating a new RFI.
    id: Optional[UUID] = None
    # ``true``: This RFI type is the default for the project.``false``: (default) This RFI type is not the default.
    is_default: Optional[bool] = None
    # The default text for the **Location** field when creating a new RFI.Note that the API does not auto-populate this value.Clients are responsible for applying the default if desired.
    location_description: Optional[str] = None
    # A list of users, companies, or roles who can be assigned to this workflow role.
    manager: Optional[list[RfiTypesGetResponse_results_manager]] = None
    # The name of the RFI type, as configured by the project admin. This name is shown in the ACC UI and in the API when selecting an RFI type.Max length: 50
    name: Optional[str] = None
    # The default priority for new RFIs of this type.The available priority values are configured in Project Admin.If no default is set, this field is ``null``.Note that the API does not auto-populate this value when creating an RFI.Clients are responsible for applying the default if desired.Some possible values: ``null``, ``High``, ``Normal``, ``Low``.
    priority: Optional[str] = None
    # A list of users, companies, or roles that can be assigned to this project role.
    project_coordinator: Optional[list[RfiTypesGetResponse_results_projectCoordinator]] = None
    # A list of users, companies, or roles that can be assigned to this project role.
    project_reviewer: Optional[list[RfiTypesGetResponse_results_projectReviewer]] = None
    # The default value for the Reference field when creating a new RFI.This is typically used when the RFI was created in another system.Note that the API does not auto-populate this value.Clients are responsible for applying the default if desired.Max length: 1000
    reference: Optional[str] = None
    # The default schedule impact value for new RFIs of this type.Possible values: ``null``, ``Yes``, ``No``, ``Unknown``.
    schedule_impact: Optional[str] = None
    # The current status of the RFI type.Possible values:- ``active``: The type is available for use when creating or updating RFIs.- ``inactive``: The type exists but cannot currently be selected when creating RFIs.- ``hidden``: The type is hidden from users in the UI but may still appear in the API.Only active types are available by default when creating new RFIs.
    status: Optional[RfiTypesGetResponse_results_status] = None
    # A list of users, companies, or roles that can be assigned to this project role.
    watchers: Optional[list[RfiTypesGetResponse_results_watchers]] = None
    # The workflow type used for this RFI type.Possible values:- ``US``: The US-style workflow, with a Reviewer and optional Manager.- ``EU``: The EU-style workflow, with a Project Coordinator and Project Reviewer.The workflow type determines the available statuses and workflow roles for RFIs of this type.
    wf_type: Optional[RfiTypesGetResponse_results_wfType] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RfiTypesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RfiTypesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RfiTypesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .rfi_types_get_response_results_manager import RfiTypesGetResponse_results_manager
        from .rfi_types_get_response_results_project_coordinator import RfiTypesGetResponse_results_projectCoordinator
        from .rfi_types_get_response_results_project_reviewer import RfiTypesGetResponse_results_projectReviewer
        from .rfi_types_get_response_results_status import RfiTypesGetResponse_results_status
        from .rfi_types_get_response_results_watchers import RfiTypesGetResponse_results_watchers
        from .rfi_types_get_response_results_wf_type import RfiTypesGetResponse_results_wfType

        from .rfi_types_get_response_results_manager import RfiTypesGetResponse_results_manager
        from .rfi_types_get_response_results_project_coordinator import RfiTypesGetResponse_results_projectCoordinator
        from .rfi_types_get_response_results_project_reviewer import RfiTypesGetResponse_results_projectReviewer
        from .rfi_types_get_response_results_status import RfiTypesGetResponse_results_status
        from .rfi_types_get_response_results_watchers import RfiTypesGetResponse_results_watchers
        from .rfi_types_get_response_results_wf_type import RfiTypesGetResponse_results_wfType

        fields: dict[str, Callable[[Any], None]] = {
            "bridgeTargetProjectIds": lambda n : setattr(self, 'bridge_target_project_ids', n.get_collection_of_primitive_values(str)),
            "category": lambda n : setattr(self, 'category', n.get_collection_of_primitive_values(str)),
            "costImpact": lambda n : setattr(self, 'cost_impact', n.get_str_value()),
            "discipline": lambda n : setattr(self, 'discipline', n.get_collection_of_primitive_values(str)),
            "dueDateOffset": lambda n : setattr(self, 'due_date_offset', n.get_int_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isDefault": lambda n : setattr(self, 'is_default', n.get_bool_value()),
            "locationDescription": lambda n : setattr(self, 'location_description', n.get_str_value()),
            "manager": lambda n : setattr(self, 'manager', n.get_collection_of_object_values(RfiTypesGetResponse_results_manager)),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "priority": lambda n : setattr(self, 'priority', n.get_str_value()),
            "projectCoordinator": lambda n : setattr(self, 'project_coordinator', n.get_collection_of_object_values(RfiTypesGetResponse_results_projectCoordinator)),
            "projectReviewer": lambda n : setattr(self, 'project_reviewer', n.get_collection_of_object_values(RfiTypesGetResponse_results_projectReviewer)),
            "reference": lambda n : setattr(self, 'reference', n.get_str_value()),
            "scheduleImpact": lambda n : setattr(self, 'schedule_impact', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(RfiTypesGetResponse_results_status)),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_object_values(RfiTypesGetResponse_results_watchers)),
            "wfType": lambda n : setattr(self, 'wf_type', n.get_enum_value(RfiTypesGetResponse_results_wfType)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("bridgeTargetProjectIds", self.bridge_target_project_ids)
        writer.write_collection_of_primitive_values("category", self.category)
        writer.write_str_value("costImpact", self.cost_impact)
        writer.write_collection_of_primitive_values("discipline", self.discipline)
        writer.write_int_value("dueDateOffset", self.due_date_offset)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isDefault", self.is_default)
        writer.write_str_value("locationDescription", self.location_description)
        writer.write_collection_of_object_values("manager", self.manager)
        writer.write_str_value("name", self.name)
        writer.write_str_value("priority", self.priority)
        writer.write_collection_of_object_values("projectCoordinator", self.project_coordinator)
        writer.write_collection_of_object_values("projectReviewer", self.project_reviewer)
        writer.write_str_value("reference", self.reference)
        writer.write_str_value("scheduleImpact", self.schedule_impact)
        writer.write_enum_value("status", self.status)
        writer.write_collection_of_object_values("watchers", self.watchers)
        writer.write_enum_value("wfType", self.wf_type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .projects_post_response_classification import ProjectsPostResponse_classification
    from .projects_post_response_platform import ProjectsPostResponse_platform
    from .projects_post_response_products import ProjectsPostResponse_products
    from .projects_post_response_project_value import ProjectsPostResponse_projectValue
    from .projects_post_response_status import ProjectsPostResponse_status

@dataclass
class ProjectsPostResponse(Parsable):
    # The account ID associated with the project.
    account_id: Optional[UUID] = None
    # The first line of the project’s address.Max length: 255
    address_line1: Optional[str] = None
    # Additional address details for the project location.Max length: 255
    address_line2: Optional[str] = None
    # Not relevantMax length: 25
    admin_group_id: Optional[str] = None
    # The ID of the business unit that the project is associated with.
    business_unit_id: Optional[UUID] = None
    # The city wher the project is located.Max length: 255
    city: Optional[str] = None
    # The classification of the project. Possible values:- ``production`` – Standard project.- ``template`` – A project that serves as a template for creating new projects.- ``component`` – A placeholder project containing reusable components (e.g., forms). Only one component project is allowed per account. Known as a library in the ACC UI.- ``sample`` – A single sample project automatically created for ACC trials (limited to one per account).
    classification: Optional[ProjectsPostResponse_classification] = None
    # The total number of companies associated with the project.Note that this field is relevant only in responses. It is ignored in requests.
    company_count: Optional[int] = None
    # The type of construction for the project. Recommended values: ``New Construction``, ``Renovation``. Any value is accepted.
    construction_type: Optional[str] = None
    # The type of contract for the project. For example, ``Lump Sum``, ``Cost Plus``, ``Guaranteed Maximum Price``, ``Unit Price``. Any value is accepted.
    contract_type: Optional[str] = None
    # The country where the project is located, using an ISO 3166-1 code.Max length: 255
    country: Optional[str] = None
    # The timestamp when the project was created, in ISO 8601 format. This value is set at creation and does not change.
    created_at: Optional[datetime.datetime] = None
    # The current phase of the project. Recommended values include, ``Concept``, ``Design``, ``Bidding``, ``Planning``, ``Preconstruction``, ``Construction``, ``Commissioning``, ``Warranty``, ``Complete``, ``Facility Management``, ``Operation``, ``Strategic Definition``, ``Preparation and Brief``, ``Concept Design``, ``Developed Design``, ``Technical Design``, ``Construction``, ``Handover and Close Out`` and ``In Use``.Any value is accepted.
    current_phase: Optional[str] = None
    # The method used to deliver the project. Recommended values include ``Design-Bid-Build``, ``Construction Management (CM) at Risk``, and ``Integrated Project Delivery (IPD)``. Any value is accepted.
    delivery_method: Optional[str] = None
    # The estimated end date for the project, in ISO 8601 format.
    end_date: Optional[str] = None
    # The internally generated ID for the project.
    id: Optional[UUID] = None
    # The URL of the main image associated with the project. This field can be ``null``.Max length: 255
    image_url: Optional[str] = None
    # Not relevant - we don't currently support this field.
    job_id: Optional[UUID] = None
    # A user-defined identifier for the project. This value is assigned when the project is created and can be used to filter projects. It supports partial matches when used with ``filterTextMatch``.Max length: 100
    job_number: Optional[str] = None
    # The timestamp of the last time someone signed into the project.
    last_sign_in: Optional[datetime.datetime] = None
    # The latitude coordinate of the project location.Max length: 25
    latitude: Optional[str] = None
    # The longitude coordinate of the project location.Max length: 25
    longitude: Optional[str] = None
    # The total number of members on the project.Note that this field is relevant only in responses. It is ignored in requests.
    member_count: Optional[int] = None
    # Not relevantMax length: 25
    member_group_id: Optional[str] = None
    # The name of the project.Max length: 255
    name: Optional[str] = None
    # The APS platform where the project is stored. Possible values: ``acc``, ``bim360``.Note that this field is relevant only in responses. It is ignored in requests.
    platform: Optional[ProjectsPostResponse_platform] = None
    # The postal or ZIP code of the project location.Max length: 255
    postal_code: Optional[str] = None
    # An array of the product objects associated with the project.Note that this array is relevant only in responses. It is ignored in requests.When a project is created, every product in the same account as the project is activated for the project. You can call `PATCH users/:userId </en/docs/acc/v1/reference/http/admin-projects-projectId-users-userId-PATCH/>`_ to separately activate one or more of the returned products for each user assigned to the project.
    products: Optional[list[ProjectsPostResponse_products]] = None
    # Contains details about the estimated cost of the project, including the amount (``value``) and the currency (``currency``).
    project_value: Optional[ProjectsPostResponse_projectValue] = None
    # The total number of sheets associated with the project.Note that this field is relevant only in responses. It is ignored in requests.
    sheet_count: Optional[int] = None
    # The estimated start date for the project, in ISO 8601 format.
    start_date: Optional[str] = None
    # The state or province where the project is located. It must be a valid name or an ISO 3166-2 code. The provided state or province must exist in the country of the project.Max length: 255
    state_or_province: Optional[str] = None
    # The status of the project.Possible values: ``active``, ``pending``, ``archived`` and ``suspended``.
    status: Optional[ProjectsPostResponse_status] = None
    # The ID of the project that was used as a template to create this project.
    template_id: Optional[UUID] = None
    # The URL of the project’s thumbnail image. This field can be ``null``.Max length: 255
    thumbnail_image_url: Optional[str] = None
    # The time zone where the project is located. It must be a valid IANA time zone name from the `IANA Time Zone Database <https://www.iana.org/time-zones>`_ (e.g., ``America/New_York``). If no time zone is set, this field may be ``null``.Possible values: ``Pacific/Honolulu``, ``America/Juneau``, ``America/Los_Angeles``, ``America/Phoenix``, ``America/Denver``, ``America/Chicago``, ``America/New_York``, ``America/Indiana/Indianapolis``, ``Pacific/Pago_Pago``, ``Pacific/Midway``, ``America/Tijuana``, ``America/Chihuahua``, ``America/Mazatlan``, ``America/Guatemala``, ``America/Mexico_City``, ``America/Monterrey``, ``America/Regina``, ``America/Bogota``, ``America/Lima``, ``America/Caracas``, ``America/Halifax``, ``America/Guyana``, ``America/La_Paz``, ``America/Santiago``, ``America/St_Johns``, ``America/Sao_Paulo``, ``America/Argentina/Buenos_Aires``, ``America/Godthab``, ``Atlantic/South_Georgia``, ``Atlantic/Azores``, ``Atlantic/Cape_Verde``, ``Africa/Casablanca``, ``Europe/Dublin``, ``Europe/Lisbon``, ``Europe/London``, ``Africa/Monrovia``, ``Etc/UTC``, ``Europe/Amsterdam``, ``Europe/Belgrade``, ``Europe/Berlin``, ``Europe/Bratislava``, ``Europe/Brussels``, ``Europe/Budapest``, ``Europe/Copenhagen``, ``Europe/Ljubljana``, ``Europe/Madrid``, ``Europe/Paris``, ``Europe/Prague``, ``Europe/Rome``, ``Europe/Sarajevo``, ``Europe/Skopje``, ``Europe/Stockholm``, ``Europe/Vienna``, ``Europe/Warsaw``, ``Africa/Algiers``, ``Europe/Zagreb``, ``Europe/Athens``, ``Europe/Bucharest``, ``Africa/Cairo``, ``Africa/Harare``, ``Europe/Helsinki``, ``Europe/Istanbul``, ``Asia/Jerusalem``, ``Europe/Kiev``, ``Africa/Johannesburg``, ``Europe/Riga``, ``Europe/Sofia``, ``Europe/Tallinn``, ``Europe/Vilnius``, ``Asia/Baghdad``, ``Asia/Kuwait``, ``Europe/Minsk``, ``Africa/Nairobi``, ``Asia/Riyadh``, ``Asia/Tehran``, ``Asia/Muscat``, ``Asia/Baku``, ``Europe/Moscow``, ``Asia/Tbilisi``, ``Asia/Yerevan``, ``Asia/Kabul``, ``Asia/Karachi``, ``Asia/Tashkent``, ``Asia/Kolkata``, ``Asia/Colombo``, ``Asia/Kathmandu``, ``Asia/Almaty``, ``Asia/Dhaka``, ``Asia/Yekaterinburg``, ``Asia/Rangoon``, ``Asia/Bangkok``, ``Asia/Jakarta``, ``Asia/Novosibirsk``, ``Asia/Shanghai``, ``Asia/Chongqing``, ``Asia/Hong_Kong``, ``Asia/Krasnoyarsk``, ``Asia/Kuala_Lumpur``, ``Australia/Perth``, ``Asia/Singapore``, ``Asia/Taipei``, ``Asia/Ulaanbaatar``, ``Asia/Urumqi``, ``Asia/Irkutsk``, ``Asia/Tokyo``, ``Asia/Seoul``, ``Australia/Adelaide``, ``Australia/Darwin``, ``Australia/Brisbane``, ``Australia/Melbourne``, ``Pacific/Guam``, ``Australia/Hobart``, ``Pacific/Port_Moresby``, ``Australia/Sydney``, ``Asia/Yakutsk``, ``Pacific/Noumea``, ``Asia/Vladivostok``, ``Pacific/Auckland``, ``Pacific/Fiji``, ``Asia/Kamchatka``, ``Asia/Magadan``, ``Pacific/Majuro``, ``Pacific/Guadalcanal``, ``Pacific/Tongatapu``, ``Pacific/Apia``, ``Pacific/Fakaofo``
    timezone: Optional[str] = None
    # The type of the project. Any value is accepted, but the following are recommended:Possible values: ``Convention Center``, ``Data Center``, ``Hotel / Motel``, ``Office``, ``Parking Structure / Garage``, ``Performing Arts``, ``Restaurant``, ``Retail``, ``Stadium / Arena``, ``Theme Park``, ``Warehouse (non-manufacturing)``, ``Assisted Living / Nursing Home``, ``Hospital``, ``Medical Laboratory``, ``Medical Office``, ``OutPatient Surgery Center``, ``Court House``, ``Dormitory``, ``Education Facility``, ``Government Building``, ``Library``, ``Military Facility``, ``Museum``, ``Prison / Correctional Facility``, ``Recreation Building``, ``Religious Building``, ``Research Facility / Laboratory``, ``Multi-Family Housing``, ``Single-Family Housing``, ``Airport``, ``Bridge``, ``Canal / Waterway``, ``Dams / Flood Control / Reservoirs``, ``Harbor / River Development``, ``Rail``, ``Seaport``, ``Streets / Roads / Highways``, ``Transportation Building``, ``Tunnel``, ``Waste Water / Sewers``, ``Water Supply``, ``Manufacturing / Factory``, ``Mining Facility``, ``Oil & Gas``, ``Plant``, ``Power Plant``, ``Solar Farm``, ``Utilities``, ``Wind Farm``, ``Demonstration Project``, ``Template Project`` and ``Training Project``.Max length: 255
    type: Optional[str] = None
    # The timestamp when the project was last updated, in ISO 8601 format. This reflects changes to project fields but not updates to resources within the project.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ProjectsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ProjectsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ProjectsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .projects_post_response_classification import ProjectsPostResponse_classification
        from .projects_post_response_platform import ProjectsPostResponse_platform
        from .projects_post_response_products import ProjectsPostResponse_products
        from .projects_post_response_project_value import ProjectsPostResponse_projectValue
        from .projects_post_response_status import ProjectsPostResponse_status

        from .projects_post_response_classification import ProjectsPostResponse_classification
        from .projects_post_response_platform import ProjectsPostResponse_platform
        from .projects_post_response_products import ProjectsPostResponse_products
        from .projects_post_response_project_value import ProjectsPostResponse_projectValue
        from .projects_post_response_status import ProjectsPostResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "accountId": lambda n : setattr(self, 'account_id', n.get_uuid_value()),
            "addressLine1": lambda n : setattr(self, 'address_line1', n.get_str_value()),
            "addressLine2": lambda n : setattr(self, 'address_line2', n.get_str_value()),
            "adminGroupId": lambda n : setattr(self, 'admin_group_id', n.get_str_value()),
            "businessUnitId": lambda n : setattr(self, 'business_unit_id', n.get_uuid_value()),
            "city": lambda n : setattr(self, 'city', n.get_str_value()),
            "classification": lambda n : setattr(self, 'classification', n.get_enum_value(ProjectsPostResponse_classification)),
            "companyCount": lambda n : setattr(self, 'company_count', n.get_int_value()),
            "constructionType": lambda n : setattr(self, 'construction_type', n.get_str_value()),
            "contractType": lambda n : setattr(self, 'contract_type', n.get_str_value()),
            "country": lambda n : setattr(self, 'country', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "currentPhase": lambda n : setattr(self, 'current_phase', n.get_str_value()),
            "deliveryMethod": lambda n : setattr(self, 'delivery_method', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "imageUrl": lambda n : setattr(self, 'image_url', n.get_str_value()),
            "jobId": lambda n : setattr(self, 'job_id', n.get_uuid_value()),
            "jobNumber": lambda n : setattr(self, 'job_number', n.get_str_value()),
            "lastSignIn": lambda n : setattr(self, 'last_sign_in', n.get_datetime_value()),
            "latitude": lambda n : setattr(self, 'latitude', n.get_str_value()),
            "longitude": lambda n : setattr(self, 'longitude', n.get_str_value()),
            "memberCount": lambda n : setattr(self, 'member_count', n.get_int_value()),
            "memberGroupId": lambda n : setattr(self, 'member_group_id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "platform": lambda n : setattr(self, 'platform', n.get_enum_value(ProjectsPostResponse_platform)),
            "postalCode": lambda n : setattr(self, 'postal_code', n.get_str_value()),
            "products": lambda n : setattr(self, 'products', n.get_collection_of_object_values(ProjectsPostResponse_products)),
            "projectValue": lambda n : setattr(self, 'project_value', n.get_object_value(ProjectsPostResponse_projectValue)),
            "sheetCount": lambda n : setattr(self, 'sheet_count', n.get_int_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "stateOrProvince": lambda n : setattr(self, 'state_or_province', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ProjectsPostResponse_status)),
            "templateId": lambda n : setattr(self, 'template_id', n.get_uuid_value()),
            "thumbnailImageUrl": lambda n : setattr(self, 'thumbnail_image_url', n.get_str_value()),
            "timezone": lambda n : setattr(self, 'timezone', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("accountId", self.account_id)
        writer.write_str_value("addressLine1", self.address_line1)
        writer.write_str_value("addressLine2", self.address_line2)
        writer.write_str_value("adminGroupId", self.admin_group_id)
        writer.write_uuid_value("businessUnitId", self.business_unit_id)
        writer.write_str_value("city", self.city)
        writer.write_enum_value("classification", self.classification)
        writer.write_int_value("companyCount", self.company_count)
        writer.write_str_value("constructionType", self.construction_type)
        writer.write_str_value("contractType", self.contract_type)
        writer.write_str_value("country", self.country)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("currentPhase", self.current_phase)
        writer.write_str_value("deliveryMethod", self.delivery_method)
        writer.write_str_value("endDate", self.end_date)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("imageUrl", self.image_url)
        writer.write_uuid_value("jobId", self.job_id)
        writer.write_str_value("jobNumber", self.job_number)
        writer.write_datetime_value("lastSignIn", self.last_sign_in)
        writer.write_str_value("latitude", self.latitude)
        writer.write_str_value("longitude", self.longitude)
        writer.write_int_value("memberCount", self.member_count)
        writer.write_str_value("memberGroupId", self.member_group_id)
        writer.write_str_value("name", self.name)
        writer.write_enum_value("platform", self.platform)
        writer.write_str_value("postalCode", self.postal_code)
        writer.write_collection_of_object_values("products", self.products)
        writer.write_object_value("projectValue", self.project_value)
        writer.write_int_value("sheetCount", self.sheet_count)
        writer.write_str_value("startDate", self.start_date)
        writer.write_str_value("stateOrProvince", self.state_or_province)
        writer.write_enum_value("status", self.status)
        writer.write_uuid_value("templateId", self.template_id)
        writer.write_str_value("thumbnailImageUrl", self.thumbnail_image_url)
        writer.write_str_value("timezone", self.timezone)
        writer.write_str_value("type", self.type)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .issues_get_response_results_linked_documents_details_position import IssuesGetResponse_results_linkedDocuments_details_position
    from .issues_get_response_results_linked_documents_details_viewable import IssuesGetResponse_results_linkedDocuments_details_viewable
    from .issues_get_response_results_linked_documents_details_viewer_state import IssuesGetResponse_results_linkedDocuments_details_viewerState

@dataclass
class IssuesGetResponse_results_linkedDocuments_details(Parsable):
    """
    Information about the individual viewable.
    """
    # An external identifier of the element the pushpin is associated with in the viewable.
    external_id: Optional[str] = None
    # The ID of the element the pushpin is associated with in the viewable.
    object_id: Optional[int] = None
    # The position of the pushpin in the viewable.
    position: Optional[IssuesGetResponse_results_linkedDocuments_details_position] = None
    # The individual viewable associated with the issue (pushpin). This is relevant for both individual 2D sheets and views within a 3D model, and individual PDF sheets within a multi-sheet PDF file. It is only relevant if the issue is associated with a file.
    viewable: Optional[IssuesGetResponse_results_linkedDocuments_details_viewable] = None
    # The viewer state at the time the pushpin was created. Maximum length: 2,500,000 characters. You can get the viewer state object by calling ViewerState.getState(). To restore the viewer instance use ViewerState.restoreState(). See the `Viewer API documentation https://developer.autodesk.com/en/docs/viewer/v2/reference/javascript/viewerstate/`_ for more details.
    viewer_state: Optional[IssuesGetResponse_results_linkedDocuments_details_viewerState] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssuesGetResponse_results_linkedDocuments_details:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssuesGetResponse_results_linkedDocuments_details
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssuesGetResponse_results_linkedDocuments_details()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issues_get_response_results_linked_documents_details_position import IssuesGetResponse_results_linkedDocuments_details_position
        from .issues_get_response_results_linked_documents_details_viewable import IssuesGetResponse_results_linkedDocuments_details_viewable
        from .issues_get_response_results_linked_documents_details_viewer_state import IssuesGetResponse_results_linkedDocuments_details_viewerState

        from .issues_get_response_results_linked_documents_details_position import IssuesGetResponse_results_linkedDocuments_details_position
        from .issues_get_response_results_linked_documents_details_viewable import IssuesGetResponse_results_linkedDocuments_details_viewable
        from .issues_get_response_results_linked_documents_details_viewer_state import IssuesGetResponse_results_linkedDocuments_details_viewerState

        fields: dict[str, Callable[[Any], None]] = {
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "objectId": lambda n : setattr(self, 'object_id', n.get_int_value()),
            "position": lambda n : setattr(self, 'position', n.get_object_value(IssuesGetResponse_results_linkedDocuments_details_position)),
            "viewable": lambda n : setattr(self, 'viewable', n.get_object_value(IssuesGetResponse_results_linkedDocuments_details_viewable)),
            "viewerState": lambda n : setattr(self, 'viewer_state', n.get_object_value(IssuesGetResponse_results_linkedDocuments_details_viewerState)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("externalId", self.external_id)
        writer.write_int_value("objectId", self.object_id)
        writer.write_object_value("position", self.position)
        writer.write_object_value("viewable", self.viewable)
        writer.write_object_value("viewerState", self.viewer_state)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class IssueTypesGetResponse_results_subtypes(Parsable):
    # 3 chars pin label.Max length: 3
    code: Optional[str] = None
    # The date and time the issue was created, in ISO8601 format.
    created_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who created the issue type.
    created_by: Optional[str] = None
    # The date and time the issue type was deleted, in ISO8601 format.
    deleted_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who deleted the issue type.
    deleted_by: Optional[str] = None
    # The ID of the issue subtype.
    id: Optional[UUID] = None
    # Stated whether the issue subtype is active.
    is_active: Optional[bool] = None
    # Not relevant
    is_read_only: Optional[bool] = None
    # The ID of the parent issue type.
    issue_type_id: Optional[UUID] = None
    # Not relevant
    order_index: Optional[int] = None
    # Not relevant
    permitted_actions: Optional[list[str]] = None
    # Not relevant
    permitted_attributes: Optional[list[str]] = None
    # Max length: 250
    title: Optional[str] = None
    # The date and time the issue type was updated, in ISO8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The unique identifier of the user who updated the issue type.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssueTypesGetResponse_results_subtypes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssueTypesGetResponse_results_subtypes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssueTypesGetResponse_results_subtypes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "isReadOnly": lambda n : setattr(self, 'is_read_only', n.get_bool_value()),
            "issueTypeId": lambda n : setattr(self, 'issue_type_id', n.get_uuid_value()),
            "orderIndex": lambda n : setattr(self, 'order_index', n.get_int_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_primitive_values(str)),
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("code", self.code)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_bool_value("isReadOnly", self.is_read_only)
        writer.write_uuid_value("issueTypeId", self.issue_type_id)
        writer.write_int_value("orderIndex", self.order_index)
        writer.write_collection_of_primitive_values("permittedActions", self.permitted_actions)
        writer.write_collection_of_primitive_values("permittedAttributes", self.permitted_attributes)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .with_issue_patch_request_body_custom_attributes import WithIssuePatchRequestBody_customAttributes
    from .with_issue_patch_request_body_gps_coordinates import WithIssuePatchRequestBody_gpsCoordinates
    from .with_issue_patch_request_body_status import WithIssuePatchRequestBody_status

@dataclass
class WithIssuePatchRequestBody(Parsable):
    # The Autodesk ID of the member, role, or company you want to assign to the issue. Note that if you select an assignee ID, you also need to select a type (``assignedToType``).We do not currently provide endpoints to programmatically find the member, role, or company IDs that you are permitted to assign to the issue. We recommend using the Data Connector API to extract the permitted IDs. See the `Retrieve Available Members Roles and Companies </en/docs/acc/v1/tutorials/issues/retrieve-available-members-roles-companies>`_ tutorial for more details.
    assigned_to: Optional[str] = None
    # The type of the current assignee of this issue. Possible values: ``user``, ``company``, ``role``, ``null``. Note that if you select a type, you also need to select the assignee ID (``assignedTo``).
    assigned_to_type: Optional[str] = None
    # A list of custom attributes of the specific issue.
    custom_attributes: Optional[list[WithIssuePatchRequestBody_customAttributes]] = None
    # A brief description of the issue and its purpose. Maximum 1000 characters.
    description: Optional[str] = None
    # The due date of the issue, in ISO8601 format.
    due_date: Optional[str] = None
    # A GPS Coordinate which represents the geo location of the issue.
    gps_coordinates: Optional[WithIssuePatchRequestBody_gpsCoordinates] = None
    # The unique identifier of the subtype of the issue.
    issue_subtype_id: Optional[UUID] = None
    # The location related to the issue, provided as plain text. Maximum 250 characters.
    location_details: Optional[str] = None
    # The unique LBS (Location Breakdown Structure) identifier that relates to the issue.
    location_id: Optional[UUID] = None
    # The list of actions permitted for the user for this issue in its current state.Note that if a user with *View and assign to their company* permissions attempts to assign a user from a another company to the issue, it will return an error.Possible Values: ``assign_all`` (can assign another user from another company to the issue), ``assign_same_company`` (can only assign another user from the same company to the issue), ``clear_assignee``, ``delete``, ``add_comment``, ``add_attachment``, ``remove_attachment``.The following values are not relevant: ``add_attachment``, ``remove_attachment``.
    permitted_actions: Optional[list[str]] = None
    # States whether the issue is published. Default value: ``false`` (e.g. unpublished).
    published: Optional[bool] = None
    # The unique identifier of the type of root cause for the issue.
    root_cause_id: Optional[UUID] = None
    # Not relevant
    snapshot_has_markups: Optional[bool] = None
    # Not relevant
    snapshot_urn: Optional[str] = None
    # The start date of the issue, in ISO8601 format.
    start_date: Optional[str] = None
    # The current status of the issue. To check the available statuses for the project, call `GET users/me </en/docs/acc/v1/reference/http/issues-me-GET>`_ and check the permitted statuses list (``issue.new.permittedStatuses``). For more information about statuses, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Issues_Statuses>`_.Possible values: ``draft``, ``open``, ``pending``, ``in_progress``, ``completed``, ``in_review``, ``not_approved``, ``in_dispute``, ``closed``
    status: Optional[WithIssuePatchRequestBody_status] = None
    # The title of the issue. Maximum 100 characters.
    title: Optional[str] = None
    # The Autodesk ID of the member you want to assign as a watcher for the issue.We do not currently provide endpoints to programmatically find the member IDs that you are permitted to assign as watchers for the issue. We recommend using the Data Connector API to extract the permitted IDs. See the `Retrieve Available Members </en/docs/acc/v1/tutorials/issues/retrieve-available-members-roles-companies>`_ tutorial for more details.
    watchers: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithIssuePatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithIssuePatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithIssuePatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_issue_patch_request_body_custom_attributes import WithIssuePatchRequestBody_customAttributes
        from .with_issue_patch_request_body_gps_coordinates import WithIssuePatchRequestBody_gpsCoordinates
        from .with_issue_patch_request_body_status import WithIssuePatchRequestBody_status

        from .with_issue_patch_request_body_custom_attributes import WithIssuePatchRequestBody_customAttributes
        from .with_issue_patch_request_body_gps_coordinates import WithIssuePatchRequestBody_gpsCoordinates
        from .with_issue_patch_request_body_status import WithIssuePatchRequestBody_status

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_str_value()),
            "assignedToType": lambda n : setattr(self, 'assigned_to_type', n.get_str_value()),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(WithIssuePatchRequestBody_customAttributes)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "gpsCoordinates": lambda n : setattr(self, 'gps_coordinates', n.get_object_value(WithIssuePatchRequestBody_gpsCoordinates)),
            "issueSubtypeId": lambda n : setattr(self, 'issue_subtype_id', n.get_uuid_value()),
            "locationDetails": lambda n : setattr(self, 'location_details', n.get_str_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_primitive_values(str)),
            "published": lambda n : setattr(self, 'published', n.get_bool_value()),
            "rootCauseId": lambda n : setattr(self, 'root_cause_id', n.get_uuid_value()),
            "snapshotHasMarkups": lambda n : setattr(self, 'snapshot_has_markups', n.get_bool_value()),
            "snapshotUrn": lambda n : setattr(self, 'snapshot_urn', n.get_str_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithIssuePatchRequestBody_status)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "watchers": lambda n : setattr(self, 'watchers', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assignedTo", self.assigned_to)
        writer.write_str_value("assignedToType", self.assigned_to_type)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_str_value("description", self.description)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_object_value("gpsCoordinates", self.gps_coordinates)
        writer.write_uuid_value("issueSubtypeId", self.issue_subtype_id)
        writer.write_str_value("locationDetails", self.location_details)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_collection_of_primitive_values("permittedActions", self.permitted_actions)
        writer.write_bool_value("published", self.published)
        writer.write_uuid_value("rootCauseId", self.root_cause_id)
        writer.write_bool_value("snapshotHasMarkups", self.snapshot_has_markups)
        writer.write_str_value("snapshotUrn", self.snapshot_urn)
        writer.write_str_value("startDate", self.start_date)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("title", self.title)
        writer.write_collection_of_primitive_values("watchers", self.watchers)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .search_rfis_post_request_body_sort_field import SearchRfisPostRequestBody_sort_field
    from .search_rfis_post_request_body_sort_order import SearchRfisPostRequestBody_sort_order

@dataclass
class SearchRfisPostRequestBody_sort(Parsable):
    # The field to sort by.Possible values: ``createdAt``, ``status``, ``dueDate``, ``title``, ``location``, ``updatedAt``, ``closedAt``, ``costImpact``, ``scheduleImpact``, ``priority``, ``discipline``, ``category``, ``reference``, ``customIdentifier``, ``customAttribute``
    field: Optional[SearchRfisPostRequestBody_sort_field] = None
    # The sort order.Possible values: ``DESC``, ``ASC``
    order: Optional[SearchRfisPostRequestBody_sort_order] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SearchRfisPostRequestBody_sort:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SearchRfisPostRequestBody_sort
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SearchRfisPostRequestBody_sort()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .search_rfis_post_request_body_sort_field import SearchRfisPostRequestBody_sort_field
        from .search_rfis_post_request_body_sort_order import SearchRfisPostRequestBody_sort_order

        from .search_rfis_post_request_body_sort_field import SearchRfisPostRequestBody_sort_field
        from .search_rfis_post_request_body_sort_order import SearchRfisPostRequestBody_sort_order

        fields: dict[str, Callable[[Any], None]] = {
            "field": lambda n : setattr(self, 'field', n.get_enum_value(SearchRfisPostRequestBody_sort_field)),
            "order": lambda n : setattr(self, 'order', n.get_enum_value(SearchRfisPostRequestBody_sort_order)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("field", self.field)
        writer.write_enum_value("order", self.order)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .requirements_get_response_spec_sections_submittal_groups_submittal_group import RequirementsGetResponse_specSections_submittalGroups_submittalGroup
    from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes

@dataclass
class RequirementsGetResponse_specSections_submittalGroups(Parsable):
    # The submittal group associcated with the submittal. This is equivalent to the *Submittal group* column in the UI.Possible values: ``ACTION AND INFORMATIONAL``, ``CLOSEOUT SUBMITTALS``, ``DIVISION 01 REQUIREMENTS``, ``FIELD QUALITY CONTROL``, ``Mockups``, ``QUALITY ASSURANCE``, ``Tests And Inspections``
    submittal_group: Optional[RequirementsGetResponse_specSections_submittalGroups_submittalGroup] = None
    # The submittalGroupTypes property
    submittal_group_types: Optional[list[RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes]] = None
    # The number of submittals for the submittal group.
    total: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RequirementsGetResponse_specSections_submittalGroups:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RequirementsGetResponse_specSections_submittalGroups
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RequirementsGetResponse_specSections_submittalGroups()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .requirements_get_response_spec_sections_submittal_groups_submittal_group import RequirementsGetResponse_specSections_submittalGroups_submittalGroup
        from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes

        from .requirements_get_response_spec_sections_submittal_groups_submittal_group import RequirementsGetResponse_specSections_submittalGroups_submittalGroup
        from .requirements_get_response_spec_sections_submittal_groups_submittal_group_types import RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes

        fields: dict[str, Callable[[Any], None]] = {
            "submittalGroup": lambda n : setattr(self, 'submittal_group', n.get_enum_value(RequirementsGetResponse_specSections_submittalGroups_submittalGroup)),
            "submittalGroupTypes": lambda n : setattr(self, 'submittal_group_types', n.get_collection_of_object_values(RequirementsGetResponse_specSections_submittalGroups_submittalGroupTypes)),
            "total": lambda n : setattr(self, 'total', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("submittalGroup", self.submittal_group)
        writer.write_collection_of_object_values("submittalGroupTypes", self.submittal_group_types)
        writer.write_int_value("total", self.total)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .clash.clash_request_builder import ClashRequestBuilder
    from .docs.docs_request_builder import DocsRequestBuilder
    from .modelset.modelset_request_builder import ModelsetRequestBuilder
    from .relationship.relationship_request_builder import RelationshipRequestBuilder

class Bim360RequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new Bim360RequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360", path_parameters)
    
    @property
    def clash(self) -> ClashRequestBuilder:
        """
        The clash property
        """
        from .clash.clash_request_builder import ClashRequestBuilder

        return ClashRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def docs(self) -> DocsRequestBuilder:
        """
        The docs property
        """
        from .docs.docs_request_builder import DocsRequestBuilder

        return DocsRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def modelset(self) -> ModelsetRequestBuilder:
        """
        The modelset property
        """
        from .modelset.modelset_request_builder import ModelsetRequestBuilder

        return ModelsetRequestBuilder(self.request_adapter, self.path_parameters)
    
    @property
    def relationship(self) -> RelationshipRequestBuilder:
        """
        The relationship property
        """
        from .relationship.relationship_request_builder import RelationshipRequestBuilder

        return RelationshipRequestBuilder(self.request_adapter, self.path_parameters)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WithTaskIdClosePostRequestBody(Parsable):
    # A list of attachment IDs to duplicate from a submittal item's previous stage in the workflow. To retrieve available attachments, call `GET items/:itemId/attachments </en/docs/acc/v1/reference/http/submittals-items-itemId-attachments-GET/>`_.
    duplicate_attachments: Optional[list[str]] = None
    # The body of the response comment, containing feedback or instructions related to the task.
    response_comment: Optional[str] = None
    # The ID of the response associated with the task, linking to the specific feedback or action taken. To retrieve this ID, call `GET responses </en/docs/acc/v1/reference/http/submittals-responses-GET/>`_.
    response_id: Optional[str] = None
    # Not relevant
    save_attachment_drafts: Optional[list[str]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithTaskIdClosePostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithTaskIdClosePostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithTaskIdClosePostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "duplicateAttachments": lambda n : setattr(self, 'duplicate_attachments', n.get_collection_of_primitive_values(str)),
            "responseComment": lambda n : setattr(self, 'response_comment', n.get_str_value()),
            "responseId": lambda n : setattr(self, 'response_id', n.get_str_value()),
            "saveAttachmentDrafts": lambda n : setattr(self, 'save_attachment_drafts', n.get_collection_of_primitive_values(str)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("duplicateAttachments", self.duplicate_attachments)
        writer.write_str_value("responseComment", self.response_comment)
        writer.write_str_value("responseId", self.response_id)
        writer.write_collection_of_primitive_values("saveAttachmentDrafts", self.save_attachment_drafts)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class CollectionsGetResponse_results(Parsable):
    # The date and time the collection was created.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the collection.
    created_by: Optional[str] = None
    # The name of the user who created the collection.
    created_by_name: Optional[str] = None
    # The unique identifier of the collection.
    id: Optional[UUID] = None
    # The name of the collection. This corresponds to the Name column in the ACC Sheets Collections Settings UI.
    name: Optional[str] = None
    # The date and time the collection was last updated.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the collection.
    updated_by: Optional[str] = None
    # The name of the user who last updated the collection.
    updated_by_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> CollectionsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: CollectionsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return CollectionsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdByName": lambda n : setattr(self, 'created_by_name', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_str_value("createdByName", self.created_by_name)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .rfis_post_response_architects_type import RfisPostResponse_architects_type

@dataclass
class RfisPostResponse_architects(Parsable):
    # The Autodesk ID of the architect.To find details about the user, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET>`_.
    id: Optional[str] = None
    # The type of architect. Will always be ``user``.
    type: Optional[RfisPostResponse_architects_type] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RfisPostResponse_architects:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RfisPostResponse_architects
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RfisPostResponse_architects()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .rfis_post_response_architects_type import RfisPostResponse_architects_type

        from .rfis_post_response_architects_type import RfisPostResponse_architects_type

        fields: dict[str, Callable[[Any], None]] = {
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_enum_value(RfisPostResponse_architects_type)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("id", self.id)
        writer.write_enum_value("type", self.type)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_attribute_patch_request_body_possible_values_updated_attributes import WithAttributePatchRequestBody_possibleValues_updatedAttributes

@dataclass
class WithAttributePatchRequestBody_possibleValues(Parsable):
    # Deletes possible values from the attribute. Each item is the ID of a possible value to delete.
    deleted_attributes: Optional[list[str]] = None
    # Adds new possible values to the attribute.Each item is the name of a new possible value as shown in the UI.
    new_attributes: Optional[list[str]] = None
    # Updates the names of existing possible values.
    updated_attributes: Optional[list[WithAttributePatchRequestBody_possibleValues_updatedAttributes]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithAttributePatchRequestBody_possibleValues:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithAttributePatchRequestBody_possibleValues
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithAttributePatchRequestBody_possibleValues()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_attribute_patch_request_body_possible_values_updated_attributes import WithAttributePatchRequestBody_possibleValues_updatedAttributes

        from .with_attribute_patch_request_body_possible_values_updated_attributes import WithAttributePatchRequestBody_possibleValues_updatedAttributes

        fields: dict[str, Callable[[Any], None]] = {
            "deletedAttributes": lambda n : setattr(self, 'deleted_attributes', n.get_collection_of_primitive_values(str)),
            "newAttributes": lambda n : setattr(self, 'new_attributes', n.get_collection_of_primitive_values(str)),
            "updatedAttributes": lambda n : setattr(self, 'updated_attributes', n.get_collection_of_object_values(WithAttributePatchRequestBody_possibleValues_updatedAttributes)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("deletedAttributes", self.deleted_attributes)
        writer.write_collection_of_primitive_values("newAttributes", self.new_attributes)
        writer.write_collection_of_object_values("updatedAttributes", self.updated_attributes)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .folders_get_response import FoldersGetResponse
    from .get_sort_query_parameter_type import GetSortQueryParameterType

class FoldersRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/transmittals/v1/projects/{projectId}/transmittals/{transmittalId}/folders
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new FoldersRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/transmittals/v1/projects/{projectId}/transmittals/{transmittalId}/folders{?limit*,offset*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[FoldersRequestBuilderGetQueryParameters]] = None) -> Optional[FoldersGetResponse]:
        """
        Retrieves all folders associated with the documents included in a specific transmittal.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[FoldersGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .folders_get_response import FoldersGetResponse

        return await self.request_adapter.send_async(request_info, FoldersGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[FoldersRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all folders associated with the documents included in a specific transmittal.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> FoldersRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: FoldersRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return FoldersRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class FoldersRequestBuilderGetQueryParameters():
        """
        Retrieves all folders associated with the documents included in a specific transmittal.
        """
        # The maximum number of results to return per page.Acceptable values: 1-200.Default value: 20.For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The index from which the response starts returning results.Default value: 0.For example, to skip the first three results, use ``offset=3``.
        offset: Optional[int] = None

        # Sorts the folders by a supported field and order.By default, folders are sorted in ascending order by ``name`` (``name asc``).To sort in descending order, add ``desc`` after the field name.Format: ``sort=<field> [asc or desc]``Supported fields: ``name``, ``lastUpdatedAt``, ``updatedByName``.Examples:- ``sort=name asc`` – sorts folders alphabetically by name.- ``sort=lastUpdatedAt desc`` – sorts folders by last updated time in descending order.Possible values: ``name``, ``lastUpdatedAt``, ``updatedByName``, ``name asc``, ``lastUpdatedAt asc``, ``updatedByName asc``, ``name desc``, ``lastUpdatedAt desc``, ``updatedByName desc``
        sort: Optional[GetSortQueryParameterType] = None

    
    @dataclass
    class FoldersRequestBuilderGetRequestConfiguration(RequestConfiguration[FoldersRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class WorkflowsPostRequestBody_steps_candidates_users(Parsable):
    # The Autodesk ID of the user. To find the Autodesk IDs of users for a project, call `GET projects/users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.
    autodesk_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WorkflowsPostRequestBody_steps_candidates_users:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WorkflowsPostRequestBody_steps_candidates_users
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WorkflowsPostRequestBody_steps_candidates_users()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "autodeskId": lambda n : setattr(self, 'autodesk_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("autodeskId", self.autodesk_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class SheetsBatchUpdatePostRequestBody_updates(Parsable):
    # The tags to be attached to the sheets.- The max length is 100.- The tags should not be space only.- After applying addTags and removeTags, the number of remain tags of the sheets should not exceed 100.The API will format the tags in the following ways before applying them to the sheets:- Remove spaces at the end and beginning of the tags.- Reduce multiple continuous spaces to a single space.- The tags are case insensitive. Upper case letters will be transformed to lower case.
    add_tags: Optional[list[str]] = None
    # The new sheet number.Only available for single sheet update.- The number should not contain these reserved characters: ``<``, ``>``, ``:``, ``````, ``/``, ````, ``|``, ``?``, ``*``, ``n``, ``r``, ``t``, ``0``, ``f``, ``'``.- You cannot assign the following reserved names to the number: CON, PRN, AUX, NUL, COM1, COM2, COM3, COM4, COM5, COM6, COM7, COM8, COM9, LPT1, LPT2, LPT3, LPT4, LPT5, LPT6, LPT7, LPT8, and LPT9.- You cannot put a period at the end of the number.- The number should not be space only.- The max length is 255.The API will format the number in the following ways before applying it to the sheet:- Remove spaces at the end and beginning of the number.- Reduce multiple continuous spaces to a single space.Max length: 255
    number: Optional[str] = None
    # The tags to be detached from the sheets.- The max length is 100.- The tags should not be space only.The API will format the tags in the following ways before comparing them to the existing tags:- Remove spaces at the end and beginning of the tags.- Reduce multiple continuous spaces to a single space.- The tags are case insensitive. Upper case letters will be transformed to lower case.
    remove_tags: Optional[list[str]] = None
    # The new title of the sheet.Only available for single sheet.- The title should not be space only.- The max length is 255.The API will format the title in the following ways before applying it to the sheet:- Remove spaces at the end and beginning of the title.- Reduce multiple continuous spaces to a single space.Max length: 255
    title: Optional[str] = None
    # The ID of the new version set.
    version_set_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SheetsBatchUpdatePostRequestBody_updates:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SheetsBatchUpdatePostRequestBody_updates
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SheetsBatchUpdatePostRequestBody_updates()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "addTags": lambda n : setattr(self, 'add_tags', n.get_collection_of_primitive_values(str)),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "removeTags": lambda n : setattr(self, 'remove_tags', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "versionSetId": lambda n : setattr(self, 'version_set_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("addTags", self.add_tags)
        writer.write_str_value("number", self.number)
        writer.write_collection_of_primitive_values("removeTags", self.remove_tags)
        writer.write_str_value("title", self.title)
        writer.write_str_value("versionSetId", self.version_set_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class PerformanceTrackingItemInstancesPatchRequestBody(Parsable):
    # The overriding output quantity that you specified to reflect a change (if any) in the scope of the tracking item instance. This value will be used instead of the value of ``outputQuantity`` in future performance tracking and forecasting for this tracking item instance.
    adjusted_output_quantity: Optional[float] = None
    # The input quantity of the tracking item instance. You can multiply this by the value of ``inputUnitPrice`` to determine the planned total for this instance.
    input_quantity: Optional[float] = None
    # The input unit of measurement of the tracking item instance. This value should always be ``hr``.
    input_unit: Optional[str] = None
    # The input unit price of the tracking item instance. You can multiply this by the value of ``inputQuantity`` to determine the planned total for this instance.
    input_unit_price: Optional[float] = None
    # A list of the IDs of the project locations where this item applies.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    locations: Optional[list[str]] = None
    # The locked performance tracking item instance field. You can lock the performance tracking item instance's planned total (``plannedTotal``) when calculating a item instance if you're updating a single value.
    locked_fields: Optional[str] = None
    # The user-provided name of the tracking item instance.
    name: Optional[str] = None
    # The user-provided or system generated code representing the tracking item instance.
    number: Optional[str] = None
    # The output quantity of the tracking item instance. You can multiply this by the value of ``outputUnitPrice`` to determine the planned total for this instance.
    output_quantity: Optional[float] = None
    # The output unit of measurement of the tracking item instance.
    output_unit: Optional[str] = None
    # The output unit price of the tracking item instance. You can multiply this by the value of ``outputQuantity`` to determine the planned total for this instance.
    output_unit_price: Optional[float] = None
    # The reported hours worked so far on the tracking item instance.
    tracked_input_quantity: Optional[float] = None
    # The reported quantity of material used so far on the tracking item instance
    tracked_output_quantity: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PerformanceTrackingItemInstancesPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PerformanceTrackingItemInstancesPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PerformanceTrackingItemInstancesPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "adjustedOutputQuantity": lambda n : setattr(self, 'adjusted_output_quantity', n.get_float_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "inputUnit": lambda n : setattr(self, 'input_unit', n.get_str_value()),
            "inputUnitPrice": lambda n : setattr(self, 'input_unit_price', n.get_float_value()),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "lockedFields": lambda n : setattr(self, 'locked_fields', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "outputQuantity": lambda n : setattr(self, 'output_quantity', n.get_float_value()),
            "outputUnit": lambda n : setattr(self, 'output_unit', n.get_str_value()),
            "outputUnitPrice": lambda n : setattr(self, 'output_unit_price', n.get_float_value()),
            "trackedInputQuantity": lambda n : setattr(self, 'tracked_input_quantity', n.get_float_value()),
            "trackedOutputQuantity": lambda n : setattr(self, 'tracked_output_quantity', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("adjustedOutputQuantity", self.adjusted_output_quantity)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("inputUnit", self.input_unit)
        writer.write_float_value("inputUnitPrice", self.input_unit_price)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("lockedFields", self.locked_fields)
        writer.write_str_value("name", self.name)
        writer.write_str_value("number", self.number)
        writer.write_float_value("outputQuantity", self.output_quantity)
        writer.write_str_value("outputUnit", self.output_unit)
        writer.write_float_value("outputUnitPrice", self.output_unit_price)
        writer.write_float_value("trackedInputQuantity", self.tracked_input_quantity)
        writer.write_float_value("trackedOutputQuantity", self.tracked_output_quantity)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .clashes_assign_post_request_body_assigned_to_type import ClashesAssignPostRequestBody_assignedToType
    from .clashes_assign_post_request_body_custom_attributes import ClashesAssignPostRequestBody_customAttributes
    from .clashes_assign_post_request_body_pushpin import ClashesAssignPostRequestBody_pushpin
    from .clashes_assign_post_request_body_status import ClashesAssignPostRequestBody_status
    from .clashes_assign_post_request_body_view_context import ClashesAssignPostRequestBody_viewContext

@dataclass
class ClashesAssignPostRequestBody(Parsable):
    # The user, role or company that the new clash group is assigned to.
    assigned_to: Optional[str] = None
    # Specifies the type that the assignedTo parameter refers to. Possible values: ``User``, ``Role``, ``Company``.
    assigned_to_type: Optional[ClashesAssignPostRequestBody_assignedToType] = None
    # The clashes to be included in the new assigned clash group. Min items: 1 Max items: 1000.
    clashes: Optional[list[int]] = None
    # The list of attributes to associate with the new issue. Max items: 64.
    custom_attributes: Optional[list[ClashesAssignPostRequestBody_customAttributes]] = None
    # The description of the new assigned clash group. Max length: 10000.
    description: Optional[str] = None
    # A document or seed file version URN to associate the issue with. Min length: 1 Max length: 80.
    document_version_urn: Optional[str] = None
    # The date and time that the new assigned clash group is due.
    due_date: Optional[datetime.datetime] = None
    # The unique identifier of the new assigned clash group.
    id: Optional[UUID] = None
    # The issue sub-type ID associated with the new assigned clash group.
    issue_sub_type_id: Optional[UUID] = None
    # The issue type ID associated with the new assigned clash group.
    issue_type_id: Optional[UUID] = None
    # The description of the location associated with the new issue.
    location_description: Optional[str] = None
    # The location ID associated with the new issue.
    location_id: Optional[UUID] = None
    # The user who owns the new issue.
    owner: Optional[str] = None
    # An issue push pin object, which describes a visual marker to place an issue on the 3D model.
    pushpin: Optional[ClashesAssignPostRequestBody_pushpin] = None
    # The root cause ID associated with the new assigned clash group.
    root_cause_id: Optional[UUID] = None
    # The unique identifiers of screenshots associated with the new assigned clash group. Max items: 5.
    screen_shots: Optional[list[str]] = None
    # The status of the new issue. Possible values: ``Open``, ``Draft``.
    status: Optional[ClashesAssignPostRequestBody_status] = None
    # The title of the new assigned clash group. Min length: 1 Max length: 4200.
    title: Optional[str] = None
    # Provides context for when this issue is viewed. Min items: 1 Max items: 50.
    view_context: Optional[list[ClashesAssignPostRequestBody_viewContext]] = None
    # The name of the viewable in the Model Derivative manifest for the supplied version URN. Min length: 1 Max length: 430.
    viewable_name: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ClashesAssignPostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ClashesAssignPostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ClashesAssignPostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .clashes_assign_post_request_body_assigned_to_type import ClashesAssignPostRequestBody_assignedToType
        from .clashes_assign_post_request_body_custom_attributes import ClashesAssignPostRequestBody_customAttributes
        from .clashes_assign_post_request_body_pushpin import ClashesAssignPostRequestBody_pushpin
        from .clashes_assign_post_request_body_status import ClashesAssignPostRequestBody_status
        from .clashes_assign_post_request_body_view_context import ClashesAssignPostRequestBody_viewContext

        from .clashes_assign_post_request_body_assigned_to_type import ClashesAssignPostRequestBody_assignedToType
        from .clashes_assign_post_request_body_custom_attributes import ClashesAssignPostRequestBody_customAttributes
        from .clashes_assign_post_request_body_pushpin import ClashesAssignPostRequestBody_pushpin
        from .clashes_assign_post_request_body_status import ClashesAssignPostRequestBody_status
        from .clashes_assign_post_request_body_view_context import ClashesAssignPostRequestBody_viewContext

        fields: dict[str, Callable[[Any], None]] = {
            "assignedTo": lambda n : setattr(self, 'assigned_to', n.get_str_value()),
            "assignedToType": lambda n : setattr(self, 'assigned_to_type', n.get_enum_value(ClashesAssignPostRequestBody_assignedToType)),
            "clashes": lambda n : setattr(self, 'clashes', n.get_collection_of_primitive_values(int)),
            "customAttributes": lambda n : setattr(self, 'custom_attributes', n.get_collection_of_object_values(ClashesAssignPostRequestBody_customAttributes)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "documentVersionUrn": lambda n : setattr(self, 'document_version_urn', n.get_str_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "issueSubTypeId": lambda n : setattr(self, 'issue_sub_type_id', n.get_uuid_value()),
            "issueTypeId": lambda n : setattr(self, 'issue_type_id', n.get_uuid_value()),
            "locationDescription": lambda n : setattr(self, 'location_description', n.get_str_value()),
            "locationId": lambda n : setattr(self, 'location_id', n.get_uuid_value()),
            "owner": lambda n : setattr(self, 'owner', n.get_str_value()),
            "pushpin": lambda n : setattr(self, 'pushpin', n.get_object_value(ClashesAssignPostRequestBody_pushpin)),
            "rootCauseId": lambda n : setattr(self, 'root_cause_id', n.get_uuid_value()),
            "screenShots": lambda n : setattr(self, 'screen_shots', n.get_collection_of_primitive_values(str)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ClashesAssignPostRequestBody_status)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "viewContext": lambda n : setattr(self, 'view_context', n.get_collection_of_object_values(ClashesAssignPostRequestBody_viewContext)),
            "viewableName": lambda n : setattr(self, 'viewable_name', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("assignedTo", self.assigned_to)
        writer.write_enum_value("assignedToType", self.assigned_to_type)
        writer.write_collection_of_primitive_values("clashes", self.clashes)
        writer.write_collection_of_object_values("customAttributes", self.custom_attributes)
        writer.write_str_value("description", self.description)
        writer.write_str_value("documentVersionUrn", self.document_version_urn)
        writer.write_datetime_value("dueDate", self.due_date)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("issueSubTypeId", self.issue_sub_type_id)
        writer.write_uuid_value("issueTypeId", self.issue_type_id)
        writer.write_str_value("locationDescription", self.location_description)
        writer.write_uuid_value("locationId", self.location_id)
        writer.write_str_value("owner", self.owner)
        writer.write_object_value("pushpin", self.pushpin)
        writer.write_uuid_value("rootCauseId", self.root_cause_id)
        writer.write_collection_of_primitive_values("screenShots", self.screen_shots)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("title", self.title)
        writer.write_collection_of_object_values("viewContext", self.view_context)
        writer.write_str_value("viewableName", self.viewable_name)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .resources_get_response_page import ResourcesGetResponse_page
    from .resources_get_response_resources import ResourcesGetResponse_resources

@dataclass
class ResourcesGetResponse(Parsable):
    # Paging information associated with a paging response.
    page: Optional[ResourcesGetResponse_page] = None
    # A list of clash test resources.
    resources: Optional[list[ResourcesGetResponse_resources]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ResourcesGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ResourcesGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ResourcesGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .resources_get_response_page import ResourcesGetResponse_page
        from .resources_get_response_resources import ResourcesGetResponse_resources

        from .resources_get_response_page import ResourcesGetResponse_page
        from .resources_get_response_resources import ResourcesGetResponse_resources

        fields: dict[str, Callable[[Any], None]] = {
            "page": lambda n : setattr(self, 'page', n.get_object_value(ResourcesGetResponse_page)),
            "resources": lambda n : setattr(self, 'resources', n.get_collection_of_object_values(ResourcesGetResponse_resources)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("page", self.page)
        writer.write_collection_of_object_values("resources", self.resources)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .assigned_post_response_clash_data import AssignedPostResponse_clashData

@dataclass
class AssignedPostResponse(Parsable):
    # The clash data associated with a clash group.
    clash_data: Optional[list[AssignedPostResponse_clashData]] = None
    # The unique identifier of the clash test associated with the assigned clash group.
    clash_test_id: Optional[UUID] = None
    # The unique identifier of the user who created the assigned clash group.
    created_by: Optional[str] = None
    # The date and time that the assigned clash group was created.
    created_on: Optional[datetime.datetime] = None
    # The unique identifier of the assigned clash group.
    id: Optional[UUID] = None
    # The unique identifier of the issue associated with the assigned clash group.
    issue_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AssignedPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AssignedPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AssignedPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .assigned_post_response_clash_data import AssignedPostResponse_clashData

        from .assigned_post_response_clash_data import AssignedPostResponse_clashData

        fields: dict[str, Callable[[Any], None]] = {
            "clashData": lambda n : setattr(self, 'clash_data', n.get_collection_of_object_values(AssignedPostResponse_clashData)),
            "clashTestId": lambda n : setattr(self, 'clash_test_id', n.get_uuid_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "createdOn": lambda n : setattr(self, 'created_on', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "issueId": lambda n : setattr(self, 'issue_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("clashData", self.clash_data)
        writer.write_uuid_value("clashTestId", self.clash_test_id)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("createdOn", self.created_on)
        writer.write_uuid_value("id", self.id)
        writer.write_uuid_value("issueId", self.issue_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .actions_post_response_errors import ActionsPostResponse_errors

@dataclass
class ActionsPostResponse(Parsable):
    # The name of the action performed on the specified item. For example, budgetOpen on a PCO.Max length: 255
    action: Optional[str] = None
    # The object ID of the item on which the action was performed - a change order (PCO, RCO, OCO, RFQ or SCO) for example.
    association_id: Optional[str] = None
    # The type of the item on which the action was performed. Possible values: ``FormInstance``, ``OCO``, ``PCO``, ``RCO``, ``RFQ``, ``SCO``, ``Expense``, ``Contract``, ``CostPayment``, ``BudgetPayment``, ``BudgetTransfer``, ``MainContract``.
    association_type: Optional[str] = None
    # An array of errors that occurred during action execution.
    errors: Optional[list[ActionsPostResponse_errors]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ActionsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ActionsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ActionsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .actions_post_response_errors import ActionsPostResponse_errors

        from .actions_post_response_errors import ActionsPostResponse_errors

        fields: dict[str, Callable[[Any], None]] = {
            "action": lambda n : setattr(self, 'action', n.get_str_value()),
            "associationId": lambda n : setattr(self, 'association_id', n.get_str_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_str_value()),
            "errors": lambda n : setattr(self, 'errors', n.get_collection_of_object_values(ActionsPostResponse_errors)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("action", self.action)
        writer.write_str_value("associationId", self.association_id)
        writer.write_str_value("associationType", self.association_type)
        writer.write_collection_of_object_values("errors", self.errors)
    




from enum import Enum

class VersionsBatchGetPostResponse_results_processState(str, Enum):
    NEEDS_PROCESSING = "NEEDS_PROCESSING",
    PROCESSING = "PROCESSING",
    PREVIOUS_SEED_PENDING = "PREVIOUS_SEED_PENDING",
    EXTRACTION_PENDING = "EXTRACTION_PENDING",
    SPLITTING = "SPLITTING",
    PREVIOUS_DOC_PENDING = "PREVIOUS_DOC_PENDING",
    PROCESSING_ABORTING = "PROCESSING_ABORTING",
    PROCESSING_ABORTED = "PROCESSING_ABORTED",
    PROCESSING_COMPLETE = "PROCESSING_COMPLETE",
    PROCESSING_PROMOTION = "PROCESSING_PROMOTION",
    PROCESSING_COPY = "PROCESSING_COPY",
    PROCESSING_PROMOTING = "PROCESSING_PROMOTING",
    PROCESSING_COPYING = "PROCESSING_COPYING",
    PROCESSING_SUSPEND = "PROCESSING_SUSPEND",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .attributes_get_response import AttributesGetResponse
    from .attributes_post_request_body import AttributesPostRequestBody
    from .attributes_post_response import AttributesPostResponse
    from .item.with_attribute_item_request_builder import WithAttributeItemRequestBuilder

class AttributesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/rfis/v3/projects/{projectId}/attributes
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new AttributesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/rfis/v3/projects/{projectId}/attributes{?filter%5Bstatus%5D*,limit*,offset*}", path_parameters)
    
    def by_attribute_id(self,attribute_id: str) -> WithAttributeItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.rfis.v3.projects.item.attributes.item collection
        param attribute_id: The ID of the custom attribute.
        Returns: WithAttributeItemRequestBuilder
        """
        if attribute_id is None:
            raise TypeError("attribute_id cannot be null.")
        from .item.with_attribute_item_request_builder import WithAttributeItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["attributeId"] = attribute_id
        return WithAttributeItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[AttributesRequestBuilderGetQueryParameters]] = None) -> Optional[AttributesGetResponse]:
        """
        Retrieves all custom attribute definitions for a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[AttributesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .attributes_get_response import AttributesGetResponse

        return await self.request_adapter.send_async(request_info, AttributesGetResponse, None)
    
    async def post(self,body: AttributesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[AttributesPostResponse]:
        """
        Creates a custom attribute definition for a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[AttributesPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .attributes_post_response import AttributesPostResponse

        return await self.request_adapter.send_async(request_info, AttributesPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[AttributesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves all custom attribute definitions for a project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: AttributesPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a custom attribute definition for a project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> AttributesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: AttributesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return AttributesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class AttributesRequestBuilderGetQueryParameters():
        """
        Retrieves all custom attribute definitions for a project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            return original_name
        
        # Filters the response to only include custom attributes with the specified status. Possible values: ``active``, ``inactive``, ``hidden``.
        filterstatus: Optional[list[str]] = None

        # The number of RFIs to return in the response. Acceptable values: ``1–200``. Default: ``10``. For example, to limit the response to two items per page, use ``limit=2``
        limit: Optional[int] = None

        # The number of items to skip before starting to return results.For example, to begin the results from the fourth item, use ``offset=3``.
        offset: Optional[int] = None

    
    @dataclass
    class AttributesRequestBuilderGetRequestConfiguration(RequestConfiguration[AttributesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class AttributesRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class WithRfiPatchResponse_attachments_attachmentType(str, Enum):
    RfiResponse = "rfiResponse",
    RfiOfficialResponse = "rfiOfficialResponse",
    BridgeFiles = "bridgeFiles",
    RfiWebHiddenFiles = "rfiWebHiddenFiles",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .documents_get_response_association_type import DocumentsGetResponse_associationType
    from .documents_get_response_error_info import DocumentsGetResponse_errorInfo
    from .documents_get_response_status import DocumentsGetResponse_status

@dataclass
class DocumentsGetResponse(Parsable):
    # The object ID of the item associated with the actions, such as a budget, contract, or cost item.
    association_id: Optional[UUID] = None
    # The type of item to which it is associated.Possible values: ``Budget``, ``Contract``, ``ScheduleOfValue``, ``FormInstance``, ``CostItem``, ``Payment``, ``MainContract``, ``BudgetPayment``, ``Expense``, ``CostPayment``, ``ExpenseItem``, ``PaymentItem``, ``OCO``, ``RCO``, ``SCO``, ``PCO``, ``RFQ``, ``DistributionItem``, ``BudgetTransfer``, ``Fee``
    association_type: Optional[DocumentsGetResponse_associationType] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # Error information if this document failed to be generated from the template.
    error_info: Optional[DocumentsGetResponse_errorInfo] = None
    # Unique auto-generated identifier of the generated document.
    id: Optional[UUID] = None
    # Not relevant
    job_id: Optional[float] = None
    # Not relevant
    pdf_urn: Optional[str] = None
    # Autodesk ID of the recipient, who is a project user inside BIM 360 Admin.
    recipient_id: Optional[str] = None
    # Not relevant
    signed_by: Optional[str] = None
    # Current status of the document. Possible values: ``Pending``, ``Started``, ``Completed``, ``Failed``, ``PendingSign``, ``Signed``.
    status: Optional[DocumentsGetResponse_status] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The version URN of the document with docx format in the Autodesk Data Management service.
    urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> DocumentsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: DocumentsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return DocumentsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .documents_get_response_association_type import DocumentsGetResponse_associationType
        from .documents_get_response_error_info import DocumentsGetResponse_errorInfo
        from .documents_get_response_status import DocumentsGetResponse_status

        from .documents_get_response_association_type import DocumentsGetResponse_associationType
        from .documents_get_response_error_info import DocumentsGetResponse_errorInfo
        from .documents_get_response_status import DocumentsGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_enum_value(DocumentsGetResponse_associationType)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "errorInfo": lambda n : setattr(self, 'error_info', n.get_object_value(DocumentsGetResponse_errorInfo)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "jobId": lambda n : setattr(self, 'job_id', n.get_float_value()),
            "pdfUrn": lambda n : setattr(self, 'pdf_urn', n.get_str_value()),
            "recipientId": lambda n : setattr(self, 'recipient_id', n.get_str_value()),
            "signedBy": lambda n : setattr(self, 'signed_by', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(DocumentsGetResponse_status)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_enum_value("associationType", self.association_type)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_object_value("errorInfo", self.error_info)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("jobId", self.job_id)
        writer.write_str_value("pdfUrn", self.pdf_urn)
        writer.write_str_value("recipientId", self.recipient_id)
        writer.write_str_value("signedBy", self.signed_by)
        writer.write_enum_value("status", self.status)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("urn", self.urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .values_batch_update_put_request_body_tabular_values_columns_column_name import ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName

@dataclass
class ValuesBatchUpdatePutRequestBody_tabularValues_columns(Parsable):
    # The name of the column to update. Possible values:For the Work Log table:``trade`` (Crew) - use ``textVal``, ``timespan`` (Total hours) - use ``timespanVal``, ``headcount`` (workers) - use ``numberVal``, ``description`` (work performed) - use ``textVal``For the Equipment table:``item`` (equipment) - use ``textVal``, ``timespan`` (Hours used) -  use ``timespanVal``, ``quantity`` (Quantity) - use ``numberVal``, ``description`` (comment) - use ``textVal``For the Materials table:``item`` (Material) - use ``textVal``, ``quantity`` (Quantity) - use ``numberVal``, ``unit`` (Unit) - use ``textVal``, ``description`` (Comment) - use ``textVal``.
    column_name: Optional[ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName] = None
    # The attribute used for updating number columns. For example, ````numberVal``: ``42````.Each tabular field is assigned a specific value type, which you need to specify when updating the field. ``numberVal`` is relevant for updating the following columns: ``headcount`` (Workers) and ``quantity`` (Quantity).
    number_val: Optional[float] = None
    # The attribute used for updating text value columns.Each tabular field is assigned a specific value type, which you need to specify when updating the field. ``textVal`` is relevant for updating the following columns: ``description`` (Work performed), ``description`` (comment), ``item`` (Equipment), ``item`` (Material), ``trade`` (Crew), and ``unit`` (Unit).Max length: 8000
    text_val: Optional[str] = None
    # The attribute used for updating time-related columns.Each tabular field is assigned a specific value type, which you need to specify when updating the field. ``timespanVal`` is relevant for updating the following columns: ``timespan`` (Total hours), ``timespan`` (Hours used).
    timespan_val: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ValuesBatchUpdatePutRequestBody_tabularValues_columns:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ValuesBatchUpdatePutRequestBody_tabularValues_columns
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ValuesBatchUpdatePutRequestBody_tabularValues_columns()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .values_batch_update_put_request_body_tabular_values_columns_column_name import ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName

        from .values_batch_update_put_request_body_tabular_values_columns_column_name import ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName

        fields: dict[str, Callable[[Any], None]] = {
            "columnName": lambda n : setattr(self, 'column_name', n.get_enum_value(ValuesBatchUpdatePutRequestBody_tabularValues_columns_columnName)),
            "numberVal": lambda n : setattr(self, 'number_val', n.get_float_value()),
            "textVal": lambda n : setattr(self, 'text_val', n.get_str_value()),
            "timespanVal": lambda n : setattr(self, 'timespan_val', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("columnName", self.column_name)
        writer.write_float_value("numberVal", self.number_val)
        writer.write_str_value("textVal", self.text_val)
        writer.write_float_value("timespanVal", self.timespan_val)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class IssueRootCauseCategoriesGetResponse_results_rootCauses(Parsable):
    # The date and time the issue root cause was created, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the issue root cause.
    created_by: Optional[str] = None
    # The date and time the issue root cause was deleted, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    deleted_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who deleted the issue root cause.
    deleted_by: Optional[str] = None
    # The ID of the issue root cause.
    id: Optional[UUID] = None
    # States whether the root cause is active.
    is_active: Optional[bool] = None
    # Not relevant
    permitted_actions: Optional[list[str]] = None
    # Not relevant
    permitted_attributes: Optional[list[str]] = None
    # The ID of the parent issue root cause category.
    root_cause_category_id: Optional[UUID] = None
    # The title of the issue root cause.Max length: 100
    title: Optional[str] = None
    # The last date and time the issue root cause was updated, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the issue root cause.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssueRootCauseCategoriesGetResponse_results_rootCauses:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssueRootCauseCategoriesGetResponse_results_rootCauses
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssueRootCauseCategoriesGetResponse_results_rootCauses()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isActive": lambda n : setattr(self, 'is_active', n.get_bool_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_primitive_values(str)),
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_primitive_values(str)),
            "rootCauseCategoryId": lambda n : setattr(self, 'root_cause_category_id', n.get_uuid_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isActive", self.is_active)
        writer.write_collection_of_primitive_values("permittedActions", self.permitted_actions)
        writer.write_collection_of_primitive_values("permittedAttributes", self.permitted_attributes)
        writer.write_uuid_value("rootCauseCategoryId", self.root_cause_category_id)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from enum import Enum

class WithUserPatchResponse_products_access(str, Enum):
    Key = "key",
    Administrator = "administrator",
    Member = "member",
    None_ = "none",
    ProjectAdministration = "projectAdministration",
    Access = "access",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .linked_files_get_response_linked_files_results_publish_status import LinkedFilesGetResponse_linkedFiles_results_publishStatus

@dataclass
class LinkedFilesGetResponse_linkedFiles_results(Parsable):
    # The URN of the linked model.
    item_id: Optional[str] = None
    # The name of the linked model, as referenced within the host model.
    model_name: Optional[str] = None
    # Indicates whether the linked model has been published. Possible values:``published`` – The model has been published.``notPublished`` – The model has not been published.
    publish_status: Optional[LinkedFilesGetResponse_linkedFiles_results_publishStatus] = None
    # A temporary URL used to download the linked model. Signed URLs are valid for 1 hour.
    signed_url: Optional[str] = None
    # The version ID of the linked model. If the model is unpublished, this field is omitted from the response.
    version_id: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> LinkedFilesGetResponse_linkedFiles_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: LinkedFilesGetResponse_linkedFiles_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return LinkedFilesGetResponse_linkedFiles_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .linked_files_get_response_linked_files_results_publish_status import LinkedFilesGetResponse_linkedFiles_results_publishStatus

        from .linked_files_get_response_linked_files_results_publish_status import LinkedFilesGetResponse_linkedFiles_results_publishStatus

        fields: dict[str, Callable[[Any], None]] = {
            "itemId": lambda n : setattr(self, 'item_id', n.get_str_value()),
            "modelName": lambda n : setattr(self, 'model_name', n.get_str_value()),
            "publishStatus": lambda n : setattr(self, 'publish_status', n.get_enum_value(LinkedFilesGetResponse_linkedFiles_results_publishStatus)),
            "signedUrl": lambda n : setattr(self, 'signed_url', n.get_str_value()),
            "versionId": lambda n : setattr(self, 'version_id', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("itemId", self.item_id)
        writer.write_str_value("modelName", self.model_name)
        writer.write_enum_value("publishStatus", self.publish_status)
        writer.write_str_value("signedUrl", self.signed_url)
        writer.write_str_value("versionId", self.version_id)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .issue_attribute_definitions_get_response_results_data_type import IssueAttributeDefinitionsGetResponse_results_dataType
    from .issue_attribute_definitions_get_response_results_metadata import IssueAttributeDefinitionsGetResponse_results_metadata

@dataclass
class IssueAttributeDefinitionsGetResponse_results(Parsable):
    # Not relevant
    container_id: Optional[UUID] = None
    # The date and time the custom attribute was created, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    created_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who created the custom attribute.
    created_by: Optional[str] = None
    # The type of custom attribute. Possible values: ``list``, ``text``, ``paragraph``, ``numeric``.
    data_type: Optional[IssueAttributeDefinitionsGetResponse_results_dataType] = None
    # The date and time the custom attribute was deleted, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    deleted_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who deleted the custom attribute.
    deleted_by: Optional[str] = None
    # The description of the custom attribute.Max length: 500
    description: Optional[str] = None
    # The ID of the custom attribute.
    id: Optional[UUID] = None
    # The metadata object; only relevant for ``list`` custom attributes.
    metadata: Optional[IssueAttributeDefinitionsGetResponse_results_metadata] = None
    # Not relevant
    permitted_actions: Optional[list[str]] = None
    # Not relevant
    permitted_attributes: Optional[list[str]] = None
    # The title of the custom attribute.Max length: 100
    title: Optional[str] = None
    # The last date and time the custom attribute was updated, in the following format: YYYY-MM-DDThh:mm:ss.sz.
    updated_at: Optional[datetime.datetime] = None
    # The Autodesk ID of the user who last updated the custom attribute.
    updated_by: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IssueAttributeDefinitionsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IssueAttributeDefinitionsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IssueAttributeDefinitionsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .issue_attribute_definitions_get_response_results_data_type import IssueAttributeDefinitionsGetResponse_results_dataType
        from .issue_attribute_definitions_get_response_results_metadata import IssueAttributeDefinitionsGetResponse_results_metadata

        from .issue_attribute_definitions_get_response_results_data_type import IssueAttributeDefinitionsGetResponse_results_dataType
        from .issue_attribute_definitions_get_response_results_metadata import IssueAttributeDefinitionsGetResponse_results_metadata

        fields: dict[str, Callable[[Any], None]] = {
            "containerId": lambda n : setattr(self, 'container_id', n.get_uuid_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_str_value()),
            "dataType": lambda n : setattr(self, 'data_type', n.get_enum_value(IssueAttributeDefinitionsGetResponse_results_dataType)),
            "deletedAt": lambda n : setattr(self, 'deleted_at', n.get_datetime_value()),
            "deletedBy": lambda n : setattr(self, 'deleted_by', n.get_str_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "metadata": lambda n : setattr(self, 'metadata', n.get_object_value(IssueAttributeDefinitionsGetResponse_results_metadata)),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_primitive_values(str)),
            "permittedAttributes": lambda n : setattr(self, 'permitted_attributes', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_uuid_value("containerId", self.container_id)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("createdBy", self.created_by)
        writer.write_enum_value("dataType", self.data_type)
        writer.write_datetime_value("deletedAt", self.deleted_at)
        writer.write_str_value("deletedBy", self.deleted_by)
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_object_value("metadata", self.metadata)
        writer.write_collection_of_primitive_values("permittedActions", self.permitted_actions)
        writer.write_collection_of_primitive_values("permittedAttributes", self.permitted_attributes)
        writer.write_str_value("title", self.title)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_str_value("updatedBy", self.updated_by)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .segment_values_get_response import SegmentValuesGetResponse

class SegmentValuesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/segment-values
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new SegmentValuesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/segment-values{?cursorState*,filter%5Bcode%5D*,filter%5Bid%5D*,filter%5BlastModifiedSince%5D*,filter%5BoriginalCode%5D*,filter%5BparentId%5D*,filter%5BsegmentId%5D*,limit*,sort*}", path_parameters)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[SegmentValuesRequestBuilderGetQueryParameters]] = None) -> Optional[SegmentValuesGetResponse]:
        """
        Retrieves the defined segment values for the specified budget code segment.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[SegmentValuesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .segment_values_get_response import SegmentValuesGetResponse

        return await self.request_adapter.send_async(request_info, SegmentValuesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[SegmentValuesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves the defined segment values for the specified budget code segment.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> SegmentValuesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: SegmentValuesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return SegmentValuesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class SegmentValuesRequestBuilderGetQueryParameters():
        """
        Retrieves the defined segment values for the specified budget code segment.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "cursor_state":
                return "cursorState"
            if original_name == "filtercode":
                return "filter%5Bcode%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filterlast_modified_since":
                return "filter%5BlastModifiedSince%5D"
            if original_name == "filteroriginal_code":
                return "filter%5BoriginalCode%5D"
            if original_name == "filterparent_id":
                return "filter%5BparentId%5D"
            if original_name == "filtersegment_id":
                return "filter%5BsegmentId%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # A cursor token used for paginating results.This value is returned in the response when additional pages of data are available. Pass the returned `cursorState` into a subsequent request to retrieve the next page of results.The cursor token is an opaque string. Do not modify or parse its contents.
        cursor_state: Optional[str] = None

        # Returns only items that are identified by the specified codes (separated by commas). For example, ``filter[code]=``2703,2704````.Note that even single codes in number format must be enclosed in double quotes:``filter[code]=``1```` or ``filter[code]=``122. 221````.
        filtercode: Optional[list[str]] = None

        # Returns only the items that are identified by the provided list of item IDs. Separate multiple IDs with commas. For example, ``filter[id]=id1,id2``.
        filterid: Optional[list[str]] = None

        # Returns only items that were modified since the specified date and time, in ISO 8601 format. For example, ``filter[lastModifiedSince]=2020-03-01T13:00:00Z``.
        filterlast_modified_since: Optional[str] = None

        # Returns only the values of segments with the specified original codes, including delimiters. For example, ``filter[originalCode]=27-03,27-04``.
        filteroriginal_code: Optional[list[str]] = None

        # Returns only the values of segments with the specified IDs of parent segments. Separate multiple parent IDs with commas. For example, ``filter[parentId]=parentId1,parentId2``. You can also return the values of segments that have no parent by including ``filter[parentId]=blank``.
        filterparent_id: Optional[list[str]] = None

        # The budget code segment ID. Separate multiple IDs with commas. For example, ``filter[segmentId]=id1,id2``.
        filtersegment_id: Optional[list[str]] = None

        # The maximum number of records returned per page. Default: ``100``. A page may contain fewer records than the limit if there are fewer matching items or if it is the last page of results.
        limit: Optional[int] = None

        # Defines the sort order for the results. Each attribute can be sorted in ``asc`` (default) or ``desc`` order. For example, ``sort=name desc`` sorts the results by name in descending order.
        sort: Optional[str] = None

    
    @dataclass
    class SegmentValuesRequestBuilderGetRequestConfiguration(RequestConfiguration[SegmentValuesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class GetFilterTypeQueryParameterType(str, Enum):
    Estimated = "estimated",
    Proposed = "proposed",
    Submitted = "submitted",
    Approved = "approved",
    Committed = "committed",




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .reviews_post_response_archived_by import ReviewsPostResponse_archivedBy
    from .reviews_post_response_created_by import ReviewsPostResponse_createdBy
    from .reviews_post_response_next_action_by import ReviewsPostResponse_nextActionBy
    from .reviews_post_response_status import ReviewsPostResponse_status

@dataclass
class ReviewsPostResponse(Parsable):
    # Indicates whether the review has been archived.``true``: the review is archived.``false``: (default) the review is active.
    archived: Optional[bool] = None
    # The date and time when the review was archived. If the review has not been archived, this value is ``null``.
    archived_at: Optional[datetime.datetime] = None
    # Information about the user who archived the review.
    archived_by: Optional[ReviewsPostResponse_archivedBy] = None
    # The date time when the review was initiated.
    created_at: Optional[datetime.datetime] = None
    # Information about the user who initiated the review.
    created_by: Optional[ReviewsPostResponse_createdBy] = None
    # The due date of the current step.
    current_step_due_date: Optional[datetime.datetime] = None
    # The ID of the current step in the review.
    current_step_id: Optional[str] = None
    # The date time when the review was completed.
    finished_at: Optional[datetime.datetime] = None
    # The unique identifier of the review.
    id: Optional[UUID] = None
    # The name of the review.
    name: Optional[str] = None
    # Information about the claimers and candidates responsible for the current step.
    next_action_by: Optional[list[ReviewsPostResponse_nextActionBy]] = None
    # A unique, auto-incrementing number assigned to the review when it is first submitted.This ID does not change, even if the review is sent back to the initiator and goes through multiple rounds.It identifies the review within the project and reflects the order in which reviews were created.
    sequence_id: Optional[int] = None
    # The current status of the review.Possible values: ``OPEN``, ``CLOSED``, ``VOID``, ``FAILED``.
    status: Optional[ReviewsPostResponse_status] = None
    # The date time when the review was last updated.
    updated_at: Optional[datetime.datetime] = None
    # The unique identifier (UUID) of the approval workflow used to create this review.
    workflow_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewsPostResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewsPostResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewsPostResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .reviews_post_response_archived_by import ReviewsPostResponse_archivedBy
        from .reviews_post_response_created_by import ReviewsPostResponse_createdBy
        from .reviews_post_response_next_action_by import ReviewsPostResponse_nextActionBy
        from .reviews_post_response_status import ReviewsPostResponse_status

        from .reviews_post_response_archived_by import ReviewsPostResponse_archivedBy
        from .reviews_post_response_created_by import ReviewsPostResponse_createdBy
        from .reviews_post_response_next_action_by import ReviewsPostResponse_nextActionBy
        from .reviews_post_response_status import ReviewsPostResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "archived": lambda n : setattr(self, 'archived', n.get_bool_value()),
            "archivedAt": lambda n : setattr(self, 'archived_at', n.get_datetime_value()),
            "archivedBy": lambda n : setattr(self, 'archived_by', n.get_object_value(ReviewsPostResponse_archivedBy)),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "createdBy": lambda n : setattr(self, 'created_by', n.get_object_value(ReviewsPostResponse_createdBy)),
            "currentStepDueDate": lambda n : setattr(self, 'current_step_due_date', n.get_datetime_value()),
            "currentStepId": lambda n : setattr(self, 'current_step_id', n.get_str_value()),
            "finishedAt": lambda n : setattr(self, 'finished_at', n.get_datetime_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "nextActionBy": lambda n : setattr(self, 'next_action_by', n.get_collection_of_object_values(ReviewsPostResponse_nextActionBy)),
            "sequenceId": lambda n : setattr(self, 'sequence_id', n.get_int_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(ReviewsPostResponse_status)),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "workflowId": lambda n : setattr(self, 'workflow_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("archived", self.archived)
        writer.write_datetime_value("archivedAt", self.archived_at)
        writer.write_object_value("archivedBy", self.archived_by)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_object_value("createdBy", self.created_by)
        writer.write_datetime_value("currentStepDueDate", self.current_step_due_date)
        writer.write_str_value("currentStepId", self.current_step_id)
        writer.write_datetime_value("finishedAt", self.finished_at)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("name", self.name)
        writer.write_collection_of_object_values("nextActionBy", self.next_action_by)
        writer.write_int_value("sequenceId", self.sequence_id)
        writer.write_enum_value("status", self.status)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_uuid_value("workflowId", self.workflow_id)
    

